/****************************************************************************
 Module
     Gun.c

 Description
     Contains functions for controlling gun wheel speed and aim.  Also
     contains functions for sequencing the ball feeding mechanism. 

 Notes
     This module is for the project in ME 218B

 History
 When           Who     What/Why
 -------------- ---     --------
 
 02/15/10 11:19 NRM      Began Coding
****************************************************************************/
 

/*----------------------------- Include Files -----------------------------*/
#include <stdio.h>
#include <mc9s12e128.h>  // derivative information 
#include <S12E128bits.h> // E128 bit definitions  
#include <timers12.h>    // Timer Module
#include <bitdefs.h>     // BIT0HI, BIT0LO....definitions
#include "PWM.h"
#include "Gun.h"

/*----------------------------- Module Defines ----------------------------*/
#define _Ball_Capacity_ 10  // change according to how many balls we hold
#define _FIRST_DELAY_ 300     // how many ms is the first delay for the gates
#define _SECOND_DELAY_ 300     // how many ms is the 2nd delay for the gates

/*---------------------------- Module Variables ---------------------------*/
static int Balls_Left = _Ball_Capacity_; // variable for # of balls left 
                                         // in holder 
static int CurrentGateStep = 1;


/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
     Gun_Motor_Init

 Parameters
     None.

 Returns
     None.

 Description
     Initializes all the ports associated with the Gun System to INPUTS 
     or OUTPUTS.  Initializes the PWM subsystem or the Gun System.
 Notes
     PWM Port for motor is initialized in PWM module.

 Author
     Nicholas Musser, 02/15/10 
****************************************************************************/
void Gun_Motor_Init(void) 
{
    TMRS12_Init(TMRS12_RATE_1MS);     // Initialize the S12Timer Module
    DDRP = DDRP | BIT4HI;  // Set Bit 4 to output for Gate #2
    DDRP = DDRP | BIT3HI;  // Set Bit 3 to output for Gate #1
}



/****************************************************************************
 Function
     Set_Speed

 Parameters
     Percentage of maximum speed desired.  eg. for 50% feed the function 50

 Returns
     NONE.  

 Description
     User gives the function an integer value between 0 and 100% which 
     represents the pecentage of maximum attainable speed.  To stop the 
     motor give the function 0.  
     
 Notes
     The motor is driven using Drive-Coast Mode.  This allows for higher
     speeds than Drive-Break Mode. 

 Author
     Nicholas Musser, 02/15/10 
****************************************************************************/
void Set_Speed(int SPEED) 
{
    PWMDTY0 = (255*SPEED)/100;  // Set the PWM Duty Cycle to user input
}



/****************************************************************************
 Function
     Aim_Gun_Up_Down

 Parameters
     None.

 Returns
     None.  

 Description
     This function allows the user to define the launch angle of the 
     gun.  
     
 Notes
     
   
 Author
     Nicholas Musser, 02/15/10 
****************************************************************************/
void Aim_Gun_Up_Down(void) 
{   
    // 
}



/****************************************************************************
 Function
     Stop_1_Open (bottom gate)

 Parameters
     None.

 Returns
     None.  

 Description
     Sets Bit 3 of Port P to LO/HI in order to place the door lock in the
     OPEN position.
     
 Notes
     

 Author
     Nicholas Musser, 02/15/10 
****************************************************************************/
void Gate_1_Open(void) 
{   
    PTP &= BIT3LO;  // Set Bit 3 of Port P HI
    PWMDTY4 = 80;
}



/****************************************************************************
 Function
     Stop_1_Closed

 Parameters
     None.

 Returns
     None.  

 Description
     Sets Bit 3 of Port P to LO/HI in order to place the door lock in the
     CLOSED position.
     
 Notes
     

 Author
     Nicholas Musser, 02/15/10 
****************************************************************************/
void Gate_1_Closed(void) 
{   
    PTP |= BIT3HI;// Set Bit 3 of Port P LO
    PWMDTY4 = 0;
}

void Gate_1_Reset(void) 
{   
    PTP &= BIT3LO;// Set Bit 3 of Port P LO
    PWMDTY4 = 0;
}




/****************************************************************************
 Function
     Stop_2_Open (top gate)

 Parameters
     None.

 Returns
     None.  

 Description
     Sets Bit 4 of Port P to LO/HI in order to place the door lock in the
     OPEN position.
     
 Notes
     

 Author
     Nicholas Musser, 02/15/10 
****************************************************************************/
void Gate_2_Open(void) 
{   
    PTP &= BIT4LO;; // Set Bit 4 of Port P HI
    PWMDTY5 = 80;
}


/****************************************************************************
 Function
     Stop_2_Closed

 Parameters
     None.

 Returns
     None.  

 Description
     Sets Bit 4 of Port P to LO/HI in order to place the door lock in the
     CLOSED position.
     
 Notes
     

 Author
     Nicholas Musser, 02/15/10 
****************************************************************************/
void Gate_2_Closed(void) 
{   
    PTP |= BIT4HI;// Set Bit 4 of Port P LO
    PWMDTY5 = 0;
}


void Gate_2_Reset(void) 
{   
    PTP &= BIT4LO;// Set Bit 4 of Port P LO
    PWMDTY5 = 0;
}





/****************************************************************************
 Function
     Release_Ball

 Parameters
     None.

 Returns
     None.  

 Description
     Controls the OPEN/CLOSE sequence of the door locks to release one ball   
     
 Notes
     NONE.

 Author
     Nicholas Musser, 02/15/10 
****************************************************************************/
void Release_Ball(void) 
{
    static int n;  // initial time variable
    static int i = 0;  // variable use in timing loop
   
    
    SetTimer(300);
    while((TIM2_TFLG1 & _S12_C4F) == 0 )
    {
      //(void)printf("10Hz\n\r");  
    }
    
    TIM2_TFLG1 = _S12_C4F; //Clear the OC2 flag    
    
    SetTimer(300);
    while((TIM2_TFLG1 & _S12_C4F) == 0 )
    {
      //(void)printf("10Hz\n\r");  
    }
    
    TIM2_TFLG1 = _S12_C4F; //Clear the OC2 flag    
    
    SetTimer(300);
    while((TIM2_TFLG1 & _S12_C4F) == 0 )
    {
      //(void)printf("10Hz\n\r");  
    }
    
    TIM2_TFLG1 = _S12_C4F; //Clear the OC2 flag    
    
    
    Gate_1_Open();  // OPEN Stop #1
    (void)printf("Gate 1 is opened\n\r"); 
    SetTimer(300);
    while((TIM2_TFLG1 & _S12_C4F) == 0 )
    {
      //(void)printf("10Hz\n\r");  
    }
    
    TIM2_TFLG1 = _S12_C4F; //Clear the OC2 flag    
    Gate_1_Reset();
    
    //KeyStroke = getchar();
    Gate_1_Closed();  // OPEN Stop #1
    (void)printf("Gate 1 is closed\n\r"); 
    SetTimer(300);
    while((TIM2_TFLG1 & _S12_C4F) == 0 )
    {
      //(void)printf("10Hz\n\r");  
    }
    
    TIM2_TFLG1 = _S12_C4F; //Clear the OC2 flag    
    Gate_1_Reset();
    
    
    //KeyStroke = getchar();
    Gate_2_Open();  // OPEN Stop #1
    (void)printf("Gate 2 is opened\n\r"); 
    SetTimer(300);
    while((TIM2_TFLG1 & _S12_C4F) == 0 )
    {
      //(void)printf("10Hz\n\r");  
    }
    
    TIM2_TFLG1 = _S12_C4F; //Clear the OC2 flag    0
    Gate_2_Reset();
    
    //KeyStroke = getchar();
    Gate_2_Closed();  // OPEN Stop #1
    (void)printf("Gate 2 is closed\n\r"); 
    SetTimer(300);
    while((TIM2_TFLG1 & _S12_C4F) == 0 )
    {
      //(void)printf("10Hz\n\r");  
    }
    
    TIM2_TFLG1 = _S12_C4F; //Clear the OC2 flag    
    Gate_2_Reset();
    Balls_Left = Balls_Left -1; // reduce ball count by 1

}

// release ball steps
char Release_Ball2(void) 
{
    switch (CurrentGateStep)
    {
      case 1:
        Gate_1_Open();  // OPEN Stop #1
        (void)printf("Gate 1 is opened\n\r"); 
        SetTimer(300);
        CurrentGateStep++;
        return 0;
        break;
      case 2:
        if((TIM2_TFLG1 & _S12_C4F) != 0 )
        {
          TIM2_TFLG1 = _S12_C4F; //Clear the OC2 flag   
          Gate_1_Reset();
          CurrentGateStep++;
          return 0;
          break;
        }
      case 3:
        Gate_1_Closed();  // CLOSE Stop #1
        (void)printf("Gate 1 is closed\n\r"); 
        SetTimer(300);
        CurrentGateStep++;
        return 0;
        break;
      case 4:
        if((TIM2_TFLG1 & _S12_C4F) != 0 )
        {
          TIM2_TFLG1 = _S12_C4F; //Clear the OC2 flag   
          Gate_1_Reset();
          CurrentGateStep++;
          return 0;
          break;
        }

      case 5:
        Gate_2_Open();  // OPEN Stop #1
        (void)printf("Gate 2 is opened\n\r"); 
        SetTimer(300);
        CurrentGateStep++;
        return 0;
        break;
      case 6:
        if((TIM2_TFLG1 & _S12_C4F) != 0 )
        {
          TIM2_TFLG1 = _S12_C4F; //Clear the OC2 flag   
          Gate_2_Reset();
          CurrentGateStep++;
          return 0;
          break;
        }
      case 7:
        Gate_2_Closed();  // CLOSE Stop #1
        (void)printf("Gate 2 is closed\n\r"); 
        SetTimer(300);
        CurrentGateStep++;
        return 0;
        break;
      case 8:
        if((TIM2_TFLG1 & _S12_C4F) != 0 )
        {
          TIM2_TFLG1 = _S12_C4F; //Clear the OC2 flag   
          Gate_2_Reset();
          // Reset all the gates back to the beginning of releasing ball
          CurrentGateStep = 1;
          Balls_Left = Balls_Left -1; // reduce ball count by 1
          printf("before RETURN out of Release_Ball2()\r\n");
          return 1;  // Finish releasing ball
          break;
        }
    }
  //return 1;
}

        
/****************************************************************************
 Function
     Release_All_Balls

 Parameters
     None.

 Returns
     None.  

 Description
     Controls the OPEN/CLOSE sequence of the door locks to release all 
     balls.  This function is used at the end of play as a last ditch 
     effort
     
 Notes
     NONE.

 Author
     Nicholas Musser, 02/15/10 
****************************************************************************/
void Release_All_Balls(void) 
{
    while(Balls_Left > 0) // for how ever many balls are left
    {
        Release_Ball(); // Release one ball
    }  
}



/****************************************************************************
 Function
     Gun_And_Feeder_STOP

 Parameters
     None.

 Returns
     None.  

 Description
     Stop the gun wheel motor and feeder system.  This would be used if 
     we receive a STOP command from the Target Commander, or the 
     game was over. 
     
 Notes
     NONE.

 Author
     Nicholas Musser, 02/15/10 
****************************************************************************/
void Gun_And_Feeder_STOP(void) 
{
    Set_Speed(0);   // Stop wheel by setting duty cycle to 0%
    Gate_2_Closed();  // Place both gates in closed position
    Gate_1_Closed();
}

int BallLeft(void)
{
  return Balls_Left;  
}
/*------------------------------ End of file ------------------------------*/

            