/****************************************************************************
 Module
     SPI.c

 Description
     Initializes and handles the Serial Communication with the Target
     Commander.  

 Notes
     This module is for the ME 218B Project

 History
 When           Who     What/Why
 -------------- ---     --------
 
 02/15/10  NRM          Began Coding
****************************************************************************/
/*----------------------------- Include Files -----------------------------*/

#include <hidef.h>      /* common defines and macros */
#include <mc9s12e128.h>     /* derivative information */
#include <stdio.h>
#include <ADS12.h>  
#include <S12vec.h>
#include <stdlib.h>
#include <S12E128bits.h> /* E128 bit definitions  */
#include <bitdefs.h> /* BIT0HI, BIT0LO....definitions */
#include "SPI.h"
#include "SMEVENTS.h"

/*----------------------------- Module Defines ----------------------------*/



/*---------------------------- Module Variables ---------------------------*/


/*---------------------------- Module Functions ---------------------------*/

void SPIM_InitializeSPI(void);
void SPI_Transmit(unsigned char);
unsigned char SPI_Transmit_And_Receive(unsigned char command);
unsigned char Request_Beacon_Status(unsigned int);
unsigned char Request_IR_Illumination(unsigned int);
//TC_Answer_t Query_Current_Target(void);
unsigned char Query_Current_Target(void);
Event_t ResponseDecoder (unsigned char Response); 
unsigned char SPIM_TransferByte(unsigned char byteOut);

/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
     SPIM_InitializeSPI

 Parameters
     None.

 Returns
     None.  

 Description
     
     
 Notes
     

 Author
     Nicholas Musser, 02/15/10 
****************************************************************************/
void SPIM_InitializeSPI(void)
{
    /*Initialize Baud Rate to 12.21kHz */
    SPIBR |= (_S12_SPR2|_S12_SPR1|_S12_SPR0) | (_S12_SPPR2|_S12_SPPR1|_S12_SPPR0);
    //SPIBR |= (_S12_SPR2) | (_S12_SPPR2|_S12_SPPR1|_S12_SPPR0);
    
    /*Initialize Clock to Mode 3*/
    SPICR1 |= (_S12_CPOL | _S12_CPHA);

    /*Set as Master*/
    SPICR1 |= _S12_MSTR;

    /*Enable Slave Select*/
    SPICR2 |= _S12_MODFEN;
    SPICR1 |= _S12_SSOE;

    /*Enable SPI Interupts*/
    //SPICR1 |= _S12_SPIE; // Enable SPI Interrupt 
    
    /*Make sure normal mode is enabled*/
    SPICR2 &= ~_S12_SPC0;

    /*Enable SPI*/
    SPICR1 |= _S12_SPE;
    
    EnableInterrupts;
}

/****************************************************************************
 Function
     SPI_Transmit_And_Receive

 Parameters
     A number representing a specific command.

 Returns
     A number representing a specifc response to the command  

 Description
     
     
 Notes
     

 Author
     Nicholas Musser, 02/15/10 
****************************************************************************/
unsigned char SPI_Transmit_And_Receive(unsigned char command)
{
  unsigned char FirstByte, SecondByte, Garbage = 0x00, temp;
  
  temp = SPIDR;  // READ RECEIVE REGISTER

  //Wait for byte to be transfered

  //Once the data transmitting is complete, write SPIDR data register with QueryData
  while(!(SPISR & _S12_SPTEF)){}
  SPIDR = command;  // WRITE SEND REGISTER
    
  //Wait for byte to be received
  while(!(SPISR & _S12_SPIF)){}    
  
  //Once the data receiving is complete, read SPIDR data register
  FirstByte = SPIDR;  // READ RECEIVE REGISTER
    
  //Wait for byte to be transfered
  while(!(SPISR & _S12_SPTEF)){}

  //Once the data transmitting is complete, write SPIDR data register with 0xFF
  SPIDR = Garbage;
  
  //(void) printf("Garbage is %x \n\r", Garbage);

  //Wait for byte to be received
  while(!(SPISR & _S12_SPIF)){}    

  //Once the data receiving is complete, read SPIDR data register
  SecondByte = SPIDR;
  
  return SecondByte; //Return the data received
}



/****************************************************************************
 Function
     SPI_Transmit

 Parameters
     None.

 Returns
     None.  

 Description
     
     
 Notes
     

 Author
     Nicholas Musser, 02/15/10 
****************************************************************************/
void SPI_Transmit(unsigned char send)
{
    /* Clear SPTEF flag */
    while(!(SPISR & _S12_SPTEF));  // While the Transmit flag is not set
    SPIDR = send; // Query the Command Generator by sending variable "send"

}



/****************************************************************************
 Function
     Query_Current_Target

 Parameters
     None.

 Returns
     Response from Target Commander in number form.  See the TC info sheet
     for meaning. If the bot sent and invalid command it will return 0 
     indicating failure.      

 Description
     To query the Target Commander, a byte of 0xAA is sent to the Target 
     Commander. The Target Commander will process the query and during 
     the second byte of the exchange return 0xCx, where x is the number 
     of the currently active target.
     
 Notes
     

 Author
     Nicholas Musser, 02/15/10 
****************************************************************************/
//TC_Answer_t Query_Current_Target(void)
unsigned char Query_Current_Target(void)
{
    unsigned char FirstWord, SecondWord;
    unsigned char receive;  // variable for response of TC
    TC_Answer_t Answer;
    receive = SPI_Transmit_And_Receive(0xAA);
    return receive;
    
}

/****************************************************************************
 Function
     Request_IR_Illumination

 Parameters
     Give the function the number which represents the beacon you want to
     illuminate (see TC info sheet). 

 Returns
     Response from Target Commander in number form.  See the TC info sheet
     for meaning. If the bot sent and invalid command it will return 0 
     indicating failure.

 Description
     To request that the Target Commander illuminate a specific Goal 
     beacon, send a byte of 0xEx to the Target Commander, replacing ?x? 
     with the number of the Goal beacon to be illuminated. The Target 
     Commander will immediately reply with 0xFB, indicating that the 
     request has been forwarded to the field and the TC is busy waiting 
     for the field?s reply. Repeated Enable IR commands will be responded 
     to with 0xFB until the field responds to the TC. Once the Target 
     Commander receives confirmation from the field, the TC will respond 
     to the next Enable IR command with the current status of the 
     requested beacon (0x4x ? 0x7x). Between the time that an Enable 
     IR command has been received and the beacon status is returned by 
     the field, a Beacon Status command may not be issued. If a Beacon 
     Status command is received during this time, the TC will respond 
     with 0xEx.
     
 Notes
     

 Author
     Nicholas Musser, 02/15/10 
****************************************************************************/
unsigned char Request_IR_Illumination(unsigned int Beacon_Number_1)
{
    unsigned char receive1;  // variable for response of TC
    unsigned char CommandReq;
    
    switch (Beacon_Number_1)
    {
      case (0):
        CommandReq = 0xE0;
        break;
      case (1):
        CommandReq = 0xE1;
        break;
      case (2):
        CommandReq = 0xE2;
        break;
      case (3):
        CommandReq = 0xE3;
        break;
      case (4):
        CommandReq = 0xE4;
        break;
      case (5):
        CommandReq = 0xE5;
        break;
      case (6):
        CommandReq = 0xE6;
        break;
      case (7):
        CommandReq = 0xE7;
        break;
      case (8):
        CommandReq = 0xE8;
        break;
      case (9):
        CommandReq = 0xE9;
        break;
      case (10):
        CommandReq = 0xEA;
        break;
      case (11):
        CommandReq = 0xEB;
        break;
      case (12):
        CommandReq = 0xEC;
        break;
      case (13):
        CommandReq = 0xED;
        break;
      case (14):
        CommandReq = 0xEE;
        break;
      case (15):
        CommandReq = 0xEF;
        break;
    }

    // Send specific beacon btye to Targer commander and receive first response
    receive1 = SPI_Transmit_And_Receive(CommandReq);
    return receive1;  // return the response from the TC
  
}




/****************************************************************************
 Function
     Request_Beacon_Status

 Parameters
     Give the function the number which represents the beacon you want to
     illuminate (see TC info sheet).

 Returns
     Response from Target Commander in number form.  See the TC info sheet
     for meaning. If the bot sent and invalid command it will return 0 
     indicating failure.

 Description
     To request the status of a specific Goal beacon, send a byte of 0xBx 
     to the Target Commander, replacing ?x? with the number of the Goal 
     beacon whose status is to be returned. The Target Commander will 
     immediately reply with 0xFB, indicating that the request has been 
     forwarded to the field and the TC is busy waiting for the field?s 
     reply. Repeated Beacon Status commands will be responded to with 0xFB
     until the field responds to the TC. Once the Target Commander receives 
     the status information from the field, the TC will respond to the next
     Beacon Status command with the current status of the requested beacon. 
     Between the time that a Beacon Status command has been received by the 
     TC and the beacon status returned by the field, an Enable IR beacon 
     command may not be issued. If an Enable IR command is received during 
     this time, the TC will respond with 0xBx.
     
 Notes
     

 Author
     Nicholas Musser, 02/15/10 
****************************************************************************/
unsigned char Request_Beacon_Status(unsigned int Beacon_Number_2)
{
    static unsigned char receive2;  // variable for response of TC
    unsigned char CommandReq2;
    
    switch (Beacon_Number_2)
    {
      case (0):
        CommandReq2 = 0xB0;
        break;
      case (1):
        CommandReq2 = 0xB1;
        break;
      case (2):
        CommandReq2 = 0xB2;
        break;
      case (3):
        CommandReq2 = 0xB3;
        break;
      case (4):
        CommandReq2 = 0xB4;
        break;
      case (5):
        CommandReq2 = 0xB5;
        break;
      case (6):
        CommandReq2 = 0xB6;
        break;
      case (7):
        CommandReq2 = 0xB7;
        break;
      case (8):
        CommandReq2 = 0xB8;
        break;
      case (9):
        CommandReq2 = 0xB9;
        break;
      case (10):
        CommandReq2 = 0xBA;
        break;
      case (11):
        CommandReq2 = 0xBB;
        break;
      case (12):
        CommandReq2 = 0xBC;
        break;
      case (13):
        CommandReq2 = 0xBD;
        break;
      case (14):
        CommandReq2 = 0xBE;
        break;
      case (15):
        CommandReq2 = 0xBF;
        break;
    }

    // Send specific beacon btye to Targer commander and receive first response
    receive2 = SPI_Transmit_And_Receive(CommandReq2);
    return receive2;  // return the response from the TC
    
}

/****************************************************************************/
// Library of responses 
Event_t ResponseDecoder (unsigned char Response) 
{
  unsigned char FirstWord, SecondWord;
    
     FirstWord = Response & 0b11110000;
     SecondWord = Response & 0b00001111;
     
     //printf("Respond command 1st and 2nd is %x%x\r\n", FirstWord, SecondWord);
     //printf("Respond command is %x\r\n", Response);
     
     // Filter out the responses
     switch (FirstWord)
     {
        case (0xC0):
          //printf("Currently active target is %i\r\n", (int)SecondWord);
          return EV_RECIEVED_TARGET;
          break;
        case (0x40):
          printf("Beacon %i is inactive\r\n", (int)SecondWord);
          return EV_BEACON_INNACTIVE;
          break;
        case (0x50):
          printf("Beacon %i is active, but not illuminated\r\n", (int)SecondWord);
          return EV_BEACON_ACTIVE_NOT_ILLUM;
          break;
        case (0x60):
          printf("Beacon %i is active, and illuminated\r\n", (int)SecondWord);
          return EV_BEACON_ACTIVE_ILLUM;
          break;
        case (0x70):
          printf("Beacon %i is hit and dark\r\n", (int)SecondWord);
          return EV_BEACON_HIT; 
          break;
        case (0x80):
          printf("Beacon %i is hit and illuminated\r\n", (int)SecondWord);
          return EV_BEACON_HIT; //Duplicates case 0x70
          break;
        case (0xD0):
          switch (SecondWord)
          {
            case (0x0E):
              //printf("Delay, game not yet started or temporarily halted, Stop your robot\r\n");
              return EV_GAME_NOT_STARTED; 
              break;
            case (0x0A):
              //printf("Return to your starting box\r\n");
              return EV_RETURN_HOME; 
              break;
            case (0x0D):
              printf("Game Over, Red Wins\r\n");
              return EV_GAME_OVER_RED_WINS; 
              break;
            case (0x0C):
              printf("Game Over, Green Wins\r\n");
              return EV_GAME_OVER_GREEN_WINS;
              break;
          }
          break;
        case (0xF0):
          switch (SecondWord)
          {
            case (0x0F):
              printf("Offline");
              return EV_OFFLINE;
              break;
            case (0x0B):
              printf("Busy executing last command\r\n");
              return EV_COMMANDER_BUSY;
              break;
            case (0x0E):
              printf("Invalid Command Byte Received\r\n");
              return; 
              break;
          }
          break;
        case (0xB0):
          printf("Waiting for Beacon status to return\r\n");
          return;
          break;
        case (0xE0):
          printf("Waiting for Enable IR response\r\n");
          return;
          break;
        default:
          //do nothing
          printf("Garbage\r\n");
          return;
          break;
     }
}
     
/* Writes byteOut to the SPI and returns the byte received*/
unsigned char SPIM_TransferByte(unsigned char byteOut)
{
  unsigned char FirstByte, SecondByte, Garbage = 0x00, temp;
  
  temp = SPIDR;  // READ RECEIVE REGISTER

  //Wait for byte to be transfered

  //Once the data transmitting is complete, write SPIDR data register with QueryData
  while(!(SPISR & _S12_SPTEF)){}
  SPIDR = byteOut;  // WRITE SEND REGISTER
    
  //Wait for byte to be received
  while(!(SPISR & _S12_SPIF)){}    
  
  //Once the data receiving is complete, read SPIDR data register
  FirstByte = SPIDR;  // READ RECEIVE REGISTER
    
  //Wait for byte to be transfered
  while(!(SPISR & _S12_SPTEF)){}

  //Once the data transmitting is complete, write SPIDR data register with 0xFF
  SPIDR = Garbage;
  
  //(void) printf("Garbage is %x \n\r", Garbage);

  //Wait for byte to be received
  while(!(SPISR & _S12_SPIF)){}    

  //Once the data receiving is complete, read SPIDR data register
  SecondByte = SPIDR;
  
  return SecondByte; //Return the data received
} 
 
unsigned char SPI_Transmit_And_Receive2(unsigned char command)
{
  unsigned char FirstByte, SecondByte, Garbage = 0x00, temp;
  
  temp = SPIDR;  // READ RECEIVE REGISTER

  //Wait for byte to be transfered

  //Once the data transmitting is complete, write SPIDR data register with QueryData
  while(!(SPISR & _S12_SPTEF)){}
  SPIDR = command;  // WRITE SEND REGISTER

  //Wait for byte to be received
  while(!(SPISR & _S12_SPIF)){}    

  //Once the data receiving is complete, read SPIDR data register
  FirstByte = SPIDR;  // READ RECEIVE REGISTER

  //Wait for byte to be transfered
  while(!(SPISR & _S12_SPTEF)){}

  //Once the data transmitting is complete, write SPIDR data register with 0xFF
  SPIDR = Garbage;

  //(void) printf("Garbage is %x \n\r", Garbage);

  //Wait for byte to be received
  while(!(SPISR & _S12_SPIF)){}    

  //Once the data receiving is complete, read SPIDR data register
  SecondByte = SPIDR;
  
  return SecondByte; //Return the data received
}
    
    
    